@@ -1,14 +1,7 @@
-use std::any::TypeId;
 use std::io;
 
 mod format_parser;
-
-#[derive(Debug, PartialEq, Eq)]
-enum InputFormatToken<'a> {
-    Text(&'a str),
-    Type(TypeId),
-    GenericType,
-}
+use format_parser::InputFormatToken;
 
 #[derive(Debug, PartialEq, Eq)]
 pub struct InputFormat<'a> {
@@ -38,9 +31,8 @@ impl<'a> InputFormat<'a> {
                             io::ErrorKind::InvalidInput,
                             "Can not split input correctly because the consecutive placeholder",
                         ));
-                    } else {
-                        capture = true;
                     }
+                    capture = true;
                 }
                 InputFormatToken::Text(text) => {
                     if let Some(text_start_offset) = input.find(text) {
